/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package media;

import javax.microedition.midlet.MIDlet;
import mmc.StartMidlet;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.Image;
import java.io.InputStream;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import contents.GenericContents;
import contents.ReadScreen;
import de.trantor.mail.Message;
import de.trantor.mail.MimeDecoder;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.media.Player;
import javax.microedition.media.Control;
import javax.microedition.media.Manager;
import javax.microedition.media.PlayerListener;
import javax.microedition.media.control.VideoControl;
import java.io.DataInputStream;
import java.io.ByteArrayInputStream;
/**
 *
 * Player manager for audio/video
 * @author Gabriele Bianchi 
 */
public class PlayerManager implements Runnable, CommandListener, PlayerListener {

	
	Form form;
  	
  	Player player;
  	String locator;
  	Display display;
		StartMidlet midlet;
    boolean is_image;
    boolean old_implemetation = true;
    ReadScreen readerScreen;
    InputStream is = null;
    String type = "";
    MimeDecoder mime;
    int count;

	public PlayerManager(String _locator, StartMidlet _midlet, boolean isImage) {
		locator = _locator;
		midlet = _midlet;
		display = midlet.display;
        	is_image = isImage;
		form = new Form("Player Manager");
		form.setCommandListener(this);
       		if (is_image)
            		form.addCommand(GenericContents.back);
        	else
		    form.addCommand(GenericContents.stop);
		//form.addCommand(Contents.pause);
		//form.addCommand(Contents.play);
	} 
	
	public PlayerManager(StartMidlet _midlet, MimeDecoder _mime, ReadScreen _reader) {
		midlet = _midlet;
		mime = _mime;
		display = midlet.display;
		readerScreen = _reader;
		form = new Form("Player Manager");
		form.setCommandListener(this);
		old_implemetation = false;
		
	}
	

	public void run() {

   	 try {
      // since we are loading data over the network, a delay can be
      // expected
            Alert alert = new Alert("Loading. Please wait ....");
            alert.setTimeout(Alert.FOREVER);
            display.setCurrent(alert);
			if (old_implemetation)
			{
				StreamConnection fconn = (StreamConnection)Connector.open(locator);
				is = fconn.openInputStream();


				if (is_image)
				{
					try
					{
						form.append(GenericContents.displayImage(is));
						
						display.setCurrent(form);
					}
					catch (NullPointerException e)
					{
						form.deleteAll(); // clears the form of any previous controls
						display.setCurrent(GenericContents.noPhone, readerScreen);
					}
				}
				else
				{
					count = 1;
					boolean markable = true;
					if (locator.endsWith("wav"))
					{
						type = "audio/x-wav";
						markable = false;
					}
					else if (locator.endsWith("mp3"))
						type = "audio/mpeg";
					else if (locator.endsWith("mid"))
						type = "audio/mid";
					else if (locator.endsWith("mpg") || locator.endsWith("mpeg"))
					{
						type = "video/mpeg";
						count = -1; //play forever
					}
				}
			}
			//new implementation
			else
			{ //read an attachment
				is = new ByteArrayInputStream(mime.getBodyBytes());
				type = mime.getType();
				count = -1;

				if (type.startsWith("image"))
				{
					try
					{
						form.append(GenericContents.displayImage(is));
						form.addCommand(GenericContents.back);
						display.setCurrent(form);

					}
					catch (NullPointerException e)
					{
						form.deleteAll(); // clears the form of any previous controls
						display.setCurrent(GenericContents.noPhone, readerScreen);

					}
				}
				else
				{

					form.addCommand(GenericContents.stop);
					player = Manager.createPlayer(is, type);
					// a listener to handle player events like starting, closing etc

					player.addPlayerListener(this);


					player.setLoopCount(count); // play count times

					player.realize(); // realize

					player.prefetch(); // prefetch

					player.start(); // and start

				}
			}
		} catch(Exception e) {
      		System.err.println(e.getMessage());
            form.deleteAll(); // clears the form of any previous controls
      		display.setCurrent(GenericContents.noPhone, readerScreen); 
    		}
	 
  	}
	/**
	 * 
	 * @param command
	 * @param disp
	 */
	public void commandAction(Command command, Displayable disp) {
        if(disp instanceof Form) {
      // if showing form, means the media is being played
      // and the user is trying to stop or pause the player
            String id = command.getLabel().toLowerCase();
            try {
                if(id.equals("stop")) { // if stopping the media play
                    player.close(); // close the player
                    
                    display.setCurrent(readerScreen); // return to gui
                
                } else if (id.equals("back")) {
                    form.deleteAll(); // clears the form of any previous controls
                    display.setCurrent(readerScreen); // return to gui
                } 
            }catch(Exception e) {
                System.err.println(e);
                form.deleteAll(); // clears the form of any previous controls
                display.setCurrent(readerScreen);
            }
        }
  	}

	/** 
	 * Handle player events 
	 */
  	public void playerUpdate(Player player, String event, Object eventData) {

    // if the event is that the player has started, show the form
    // but only if the event data indicates that the event relates to newly
    // stated player, as the STARTED event is fired even if a player is
    // restarted. Note that eventData indicates the time at which the start
    // event is fired.
    		if ((event.equals(PlayerListener.STARTED)) && new Long(0L).equals((Long)eventData)) {

      // see if we can show a video control, depending on whether the media
      // is a video or not
      			VideoControl vc = null;
      			if((vc = (VideoControl)player.getControl("VideoControl")) != null) {
        			Item videoDisp = (Item)vc.initDisplayMode(vc.USE_GUI_PRIMITIVE, null);
        			form.append(videoDisp);
      			}

      			display.setCurrent(form);
    		} else if(event.equals(PlayerListener.CLOSED)) {

      			form.deleteAll(); // clears the form of any previous controls
    		}
  	}
	
}
