/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package mmc;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.io.SecureConnection;
import de.trantor.mail.Connection;

/**
 * Connection class
 * @author Gabriele Bianchi
 *
 */
public class MMConnection extends Connection {

    /**
     * Holds the socket connetion object (from the Generic Connection
     * Framework) that is the basis of this connection.
     */
    private StreamConnection socket;

    /**
     * Holds the InputStream associated with the socket.
     */
    private InputStream input;
    
    /**
     * Holds the OutputStream associated with the socket.
     */
    private OutputStream output;

    /**
     * Opens a connection with the given host on the given port. If the
     * ssl parameter is true, secure socket layer is used.
     *
     * @see #close
     */
    public void open(String host, int port, boolean ssl) throws IOException {
    	 System.out.println("Connecting..");
        
        if (ssl)
        	socket = (SecureConnection)Connector.open("ssl://" + host + ":" + port);
        else
        	socket = (StreamConnection)Connector.open("socket://" + host + ":" + port, Connector.READ_WRITE, true);
        System.out.println("Connected");
        input = socket.openInputStream();
        output = socket.openOutputStream();
       
    }

    /**
     * Closes the connection.
     *
     * @see #open
     */
    public void close() throws IOException {

        if (socket != null) {
            socket.close();
            socket = null;
        }
    }
    /**
     * 
     * @param buffer
     * @param offset
     * @param count
     * @return
     * @throws IOException
     */
    protected int read(byte[] buffer, int offset, int count) throws IOException {
	
        return input.read(buffer, offset, count);
    }
    /**
     * 
     * @param buffer
     * @param offset
     * @param count
     * @throws IOException
     */
    protected void write(byte[] buffer, int offset, int count) throws IOException {
	
        output.write(buffer, offset, count);
    }
    /**
     * 
     * @return
     */
    public boolean connected() {
        return (socket != null);
    }
}