/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package threads;

import java.lang.Thread;
import de.trantor.mail.*;
import java.util.Vector;
import java.io.IOException;
import mmc.MMConnection;
import util.Datas;
import mmc.StartMidlet;

/**
 * Thread for the inbox connections
 * @author Gabriele Bianchi
 *
 */
public class InboxThread extends Thread {

	private boolean isPop;
	private MMConnection conn;
	private StartMidlet midlet;
	public InboxClient inboxClient; 
	
	public InboxThread(boolean pop, StartMidlet mid) {
		isPop = pop;
		conn = new MMConnection();
		midlet = mid;
	}
	/**
	 * run thread
	 */
	public void run() {
		if (isPop)
			inboxClient = new Pop3Client(conn);
		else
			 inboxClient = new ImapClient(conn);
		try {	 
			inboxClient.open(Datas.pop3Host, Datas.port, Datas.ssl, Datas.pop3User, Datas.password);
			midlet.display.setCurrent(midlet.getGuiInboxScreen());
			Datas.connected = true;
		}catch (Exception e){ midlet.notifyBadConnection(e.getMessage());}
		
	}
	
	public void close() {
		try {
			inboxClient.close();
			System.out.println("Close connection");
			Datas.connected = false;
		}catch (Exception e){ System.out.print(e.getMessage());}
	}
	/**
	 * Get the list of mails
	 * @return vector of messages
	 * @throws MailException
	 * @throws IOException
	 */
	public Vector getMessageList() throws MailException, IOException {
        Vector list = new Vector();
		int read = 0; //limit of mail readable
        int count = inboxClient.getMessageCount();
        for (int i = count-1; i >= 0 && read < 40; i--) {
            String uid = inboxClient.getUniqueId(i);
            int size = inboxClient.getSize(i);
            System.out.println(" Message " + i + " has ID \"" + uid + "\" and is " + size + " bytes.");
            Message message = inboxClient.getHeaders(i);
            list.addElement(message.getHeaderValue("Subject", "No subject")
            + " ("
            + Message.getMachineAddress(message.getHeaderValue("From", "No sender"))
            + ")");

			read++; //segnalare se non sono state prese tutte le mail..TODO
        }
        return list;
    }
    /**
     * Get a message 
     * @param index
     * @return Message
     * @throws MailException
     * @throws IOException
     */
    public Message getMessage(int index) throws MailException, IOException {
    	return inboxClient.getMessage(index);
    }
    /**
     * Remove a message
     * @param index
     * @throws MailException
     * @throws IOException
     */
    public void removeMessage(int index) throws MailException, IOException {
    	inboxClient.removeMessage(index);
	return;
    }

	
}
